/**
 * @file json-rpc.h
 * @date 02.08.2011
 * @author Peter Spiess-Knafl
 * @brief json-rpc.h
 *
 * This file is meant to include all necessary .h files of this framework.
 */

#ifndef JSONRPCCPP_SERVER_H_
#define JSONRPCCPP_SERVER_H_

#include <jsonrpccpp/server/abstractserver.h>
#include <jsonrpccpp/common/exception.h>


#endif /* JSONRPCCPP_SERVER_H_ */
